@@ -601,6 +601,21 @@ class TestYamlMigrations:
     def test_bool(self, migration_test, setting, old, new):
         migration_test(setting, old, new)
 
+    @pytest.mark.parametrize('ssl_strict, certificate_errors', [
+        (True, 'block'),
+        (False, 'load-insecurely'),
+        ('ask', 'ask'),
+    ])
+    def test_ssl_strict(self, yaml, autoconfig, ssl_strict, certificate_errors):
+        autoconfig.write({'content.ssl_strict': {'global': ssl_strict}})
+
+        yaml.load()
+        yaml._save()
+
+        data = autoconfig.read()
+        assert 'content.ssl_strict' not in data
+        assert data['content.tls.certificate_errors']['global'] == certificate_errors
+
     @pytest.mark.parametrize('setting', [
         'tabs.title.format',
         'tabs.title.format_pinned',
@@ -744,13 +759,13 @@ class ConfPy:
 
     """Helper class to get a confpy fixture."""
 
-    def __init__(self, tmpdir, filename: str = "config.py"):
-        self._file = tmpdir / filename
+    def __init__(self, tmp_path, filename: str = "config.py"):
+        self._file = tmp_path / filename
         self.filename = str(self._file)
 
     def write(self, *lines):
         text = '\n'.join(lines)
-        self._file.write_text(text, 'utf-8', ensure=True)
+        self._file.write_text(text, 'utf-8')
 
     def read(self, error=False, warn_autoconfig=False):
         """Read the config.py via configfiles and check for errors."""
@@ -777,8 +792,8 @@ class ConfPy:
 
 
 @pytest.fixture
-def confpy(tmpdir, config_tmpdir, data_tmpdir, config_stub, key_config_stub):
-    return ConfPy(tmpdir)
+def confpy(tmp_path, config_tmpdir, data_tmpdir, config_stub, key_config_stub):
+    return ConfPy(tmp_path)
 
 
 class TestConfigPyModules:
@@ -786,8 +801,8 @@ class TestConfigPyModules:
     pytestmark = pytest.mark.usefixtures('config_stub', 'key_config_stub')
 
     @pytest.fixture
-    def qbmodulepy(self, tmpdir):
-        return ConfPy(tmpdir, filename="qbmodule.py")
+    def qbmodulepy(self, tmp_path):
+        return ConfPy(tmp_path, filename="qbmodule.py")
 
     @pytest.fixture(autouse=True)
     def restore_sys_path(self):
@@ -795,7 +810,7 @@ class TestConfigPyModules:
         yield
         sys.path = old_path
 
-    def test_bind_in_module(self, confpy, qbmodulepy, tmpdir):
+    def test_bind_in_module(self, confpy, qbmodulepy, tmp_path):
         qbmodulepy.write(
             'def run(config):',
             '    config.bind(",a", "message-info foo", mode="normal")')
@@ -804,9 +819,9 @@ class TestConfigPyModules:
         expected = {'normal': {',a': 'message-info foo'}}
         assert config.instance.get_obj('bindings.commands') == expected
         assert "qbmodule" not in sys.modules.keys()
-        assert tmpdir not in sys.path
+        assert tmp_path not in sys.path
 
-    def test_restore_sys_on_err(self, confpy, qbmodulepy, tmpdir):
+    def test_restore_sys_on_err(self, confpy, qbmodulepy, tmp_path):
         confpy.write_qbmodule()
         qbmodulepy.write('def run(config):',
                          '    1/0')
@@ -815,9 +830,9 @@ class TestConfigPyModules:
         assert error.text == "Unhandled exception"
         assert isinstance(error.exception, ZeroDivisionError)
         assert "qbmodule" not in sys.modules.keys()
-        assert tmpdir not in sys.path
+        assert tmp_path not in sys.path
 
-    def test_fail_on_nonexistent_module(self, confpy, qbmodulepy, tmpdir):
+    def test_fail_on_nonexistent_module(self, confpy, qbmodulepy, tmp_path):
         qbmodulepy.write('def run(config):',
                          '    pass')
         confpy.write('import foobar',
@@ -832,13 +847,13 @@ class TestConfigPyModules:
         assert tblines[0] == "Traceback (most recent call last):"
         assert tblines[-1].endswith("Error: No module named 'foobar'")
 
-    def test_no_double_if_path_exists(self, confpy, qbmodulepy, tmpdir):
-        sys.path.insert(0, tmpdir)
+    def test_no_double_if_path_exists(self, confpy, qbmodulepy, tmp_path):
+        sys.path.insert(0, tmp_path)
         confpy.write('import sys',
                      'if sys.path[0] in sys.path[1:]:',
                      '    raise Exception("Path not expected")')
         confpy.read()
-        assert sys.path.count(tmpdir) == 1
+        assert sys.path.count(tmp_path) == 1
 
 
 class TestConfigPy:
@@ -1004,9 +1019,9 @@ class TestConfigPy:
         confpy.read()
         assert config.instance.get_obj(option)[-1] == value
 
-    def test_oserror(self, tmpdir, data_tmpdir, config_tmpdir):
+    def test_oserror(self, tmp_path, data_tmpdir, config_tmpdir):
         with pytest.raises(configexc.ConfigFileErrors) as excinfo:
-            configfiles.read_config_py(str(tmpdir / 'foo'))
+            configfiles.read_config_py(str(tmp_path / 'foo'))
 
         assert len(excinfo.value.errors) == 1
         error = excinfo.value.errors[0]
@@ -1154,12 +1169,12 @@ class TestConfigPy:
         assert error.traceback is not None
 
     @pytest.mark.parametrize('location', ['abs', 'rel'])
-    def test_source(self, tmpdir, confpy, location):
+    def test_source(self, tmp_path, confpy, location):
         if location == 'abs':
-            subfile = tmpdir / 'subfile.py'
+            subfile = tmp_path / 'subfile.py'
             arg = str(subfile)
         else:
-            subfile = tmpdir / 'config' / 'subfile.py'
+            subfile = tmp_path / 'config' / 'subfile.py'
             arg = 'subfile.py'
 
         subfile.write_text("c.content.javascript.enabled = False",
@@ -1169,11 +1184,11 @@ class TestConfigPy:
 
         assert not config.instance.get_obj('content.javascript.enabled')
 
-    def test_source_configpy_arg(self, tmpdir, data_tmpdir, monkeypatch):
+    def test_source_configpy_arg(self, tmp_path, data_tmpdir, monkeypatch):
         alt_filename = 'alt-config.py'
 
-        alt_confpy_dir = tmpdir / 'alt-confpy-dir'
-        alt_confpy_dir.ensure(dir=True)
+        alt_confpy_dir = tmp_path / 'alt-confpy-dir'
+        alt_confpy_dir.mkdir()
         monkeypatch.setattr(standarddir, 'config_py',
                             lambda: str(alt_confpy_dir / alt_filename))
 
@@ -1187,8 +1202,8 @@ class TestConfigPy:
 
         assert not config.instance.get_obj('content.javascript.enabled')
 
-    def test_source_errors(self, tmpdir, confpy):
-        subfile = tmpdir / 'config' / 'subfile.py'
+    def test_source_errors(self, tmp_path, confpy):
+        subfile = tmp_path / 'config' / 'subfile.py'
         subfile.write_text("c.foo = 42", encoding='utf-8')
         confpy.write("config.source('subfile.py')")
         error = confpy.read(error=True)
@@ -1196,8 +1211,8 @@ class TestConfigPy:
         assert error.text == "While setting 'foo'"
         assert isinstance(error.exception, configexc.NoOptionError)
 
-    def test_source_multiple_errors(self, tmpdir, confpy):
-        subfile = tmpdir / 'config' / 'subfile.py'
+    def test_source_multiple_errors(self, tmp_path, confpy):
+        subfile = tmp_path / 'config' / 'subfile.py'
         subfile.write_text("c.foo = 42", encoding='utf-8')
         confpy.write("config.source('subfile.py')", "c.bar = 23")
 
@@ -1218,8 +1233,8 @@ class TestConfigPy:
         assert isinstance(error.exception, FileNotFoundError)
 
     @pytest.mark.parametrize('reverse', [True, False])
-    def test_source_warn_autoconfig(self, tmpdir, confpy, reverse):
-        subfile = tmpdir / 'config' / 'subfile.py'
+    def test_source_warn_autoconfig(self, tmp_path, confpy, reverse):
+        subfile = tmp_path / 'config' / 'subfile.py'
         subfile.write_text("c.content.javascript.enabled = False",
                            encoding='utf-8')
         lines = [
@@ -1383,8 +1398,8 @@ class TestConfigPyWriter:
         expected = "config.set('opt', 'ask', 'https://www.example.com/')"
         assert expected in text
 
-    def test_write(self, tmpdir):
-        pyfile = tmpdir / 'config.py'
+    def test_write(self, tmp_path):
+        pyfile = tmp_path / 'config.py'
         writer = configfiles.ConfigPyWriter(options=[], bindings={},
                                             commented=False)
         writer.write(str(pyfile))
